# Generated by Django 2.2.4 on 2020-01-03 08:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_quickbooks', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='realm',
            name='is_active',
            field=models.BooleanField(default=True),
        ),
        migrations.AlterField(
            model_name='realm',
            name='password',
            field=models.CharField(max_length=128, null=True),
        ),
        migrations.AlterField(
            model_name='realm',
            name='schema_name',
            field=models.CharField(max_length=100, unique=True),
        ),
    ]
