# Generated by Django 2.2.4 on 2020-01-17 10:01

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('django_quickbooks', '0002_auto_20200103_0801'),
    ]

    operations = [
        migrations.CreateModel(
            name='QBTask',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('qb_operation', models.CharField(max_length=25)),
                ('qb_resource', models.CharField(max_length=50)),
                ('object_id', models.CharField(max_length=50, null=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('content_type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
                ('realm', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='qb_tasks', to='django_quickbooks.Realm')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]
