# Generated by Django 2.2.9 on 2020-01-22 11:40

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('django_quickbooks', '0003_qbtask'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='QBTask',
            new_name='QBDTask',
        ),
    ]
